package API;

import com.fasterxml.jackson.databind.ObjectMapper;
import dao.ThreadDAO;
import dao.UserDAO;
import dto.Message;
import dto.MyThread;
import dto.User;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@WebServlet("/CampusTalkDatas/*")
public class ControleurRestCampusTalk extends HttpServlet {
    private final ObjectMapper objectMapper = new ObjectMapper();
    private final UserDAO userDao = new UserDAO();
    private final ThreadDAO threadDao = new ThreadDAO();

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        res.setContentType("application/json;charset=UTF-8");
        PrintWriter out = res.getWriter();

        String pathInfo = req.getPathInfo();

        List<MyThread> threadsFollowed = new ArrayList<>();

        if (pathInfo == null || pathInfo.equals("/")) {
            List<User> users = userDao.getAllUsers();
            out.println(objectMapper.writeValueAsString(users));
        } else {
            try {
                int id = Integer.parseInt(pathInfo.substring(1));
                User user = userDao.getUserById(id);
                List<Integer> threadsId = userDao.getThreadsFollowedByUser(user);


                for (Integer thId: threadsId) {
                    MyThread th = threadDao.getThreadById(thId);
                    threadsFollowed.add(th);
                }

                if (user != null) {
                        out.println(objectMapper.writeValueAsString(threadsFollowed));
                } else {
                    res.sendError(HttpServletResponse.SC_NOT_FOUND, "User not found");
                }
            } catch (NumberFormatException e) {
                res.sendError(HttpServletResponse.SC_BAD_REQUEST, "invalid ID");
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        }

        out.close();
    }
}